@@ -31,7 +31,10 @@ function ShapeRectanglePP(guiAction) {
 
     this.createPolyline = false;
     this.fill = false;
-    this.setUiOptions("../Shape.ui");
+    this.roundCorners = false;
+    this.radius = 1.0;
+
+    this.setUiOptions(["../Shape.ui", "ShapeRectanglePP.ui"]);
 }
 
 ShapeRectanglePP.prototype = new DrawBasedOnRectanglePP();
@@ -73,6 +76,29 @@ ShapeRectanglePP.prototype.slotFillChanged = function(checked) {
     Shape.slotFillChanged(this, checked);
 };
 
+ShapeRectanglePP.prototype.slotRoundCornersChanged = function(checked) {
+    Shape.slotRoundCornersChanged(this, checked);
+};
+
+ShapeRectanglePP.prototype.slotRadiusChanged = function(v) {
+    Shape.slotRadiusChanged(this, v);
+};
+
+ShapeRectanglePP.prototype.slotRotateChanged = function(v) {
+    this.rotate = v;
+    this.updatePreview(true);
+};
+
+ShapeRectanglePP.prototype.slotRotationChanged = function(value) {
+    if (isNumber(value)) {
+        this.rotation = value;
+    }
+    else {
+        this.rotation = 0.0;
+    }
+    this.updatePreview(true);
+};
+
 ShapeRectanglePP.prototype.initUiOptions = function(resume, optionsToolBar) {
     DrawBasedOnRectanglePP.prototype.initUiOptions.call(this, resume, optionsToolBar);
 
