@@ -38,6 +38,10 @@ GraphicsViewPreferences.initPreferences = function(pageWidget, calledByPrefDialo
 //    cbCapStyle.addItem(qsTr("Round Cap"), Qt.RoundCap.valueOf());
 //    cbCapStyle.addItem(qsTr("Flat Cap"), Qt.FlatCap.valueOf());
 //    cbCapStyle.addItem(qsTr("Square Cap"), Qt.SquareCap.valueOf());
+
+    widgets["Threads"].setItemText(0, widgets["Threads"].itemText(0) + " (" + Math.min(RS.getIdealThreadCount(), 6) + ")");
+
+    widgets["DefaultLineweight"].setLineweight(RLineweight.Weight000);
 };
 
 GraphicsViewPreferences.applyPreferences = function(doc, mdiChild) {
@@ -45,7 +49,8 @@ GraphicsViewPreferences.applyPreferences = function(doc, mdiChild) {
         return;
     }
 
-    var mt = RSettings.getBoolValue("GraphicsView/Multithreading", true);
+    var numThreads = RSettings.getIntValue("GraphicsView/Threads", Math.min(RS.getIdealThreadCount(), 6));
+    //EAction.handleUserMessage(qsTr("Threads:") + " " + numThreads);
 
     var di = mdiChild.getDocumentInterface();
     var scenes = di.getGraphicsScenes();
@@ -59,12 +64,9 @@ GraphicsViewPreferences.applyPreferences = function(doc, mdiChild) {
             if (!view.isPrintPreview()) {
                 view.setBackgroundColor(new QColor());
             }
-            if (mt) {
-                view.setNumThreads(RS.getIdealThreadCount());
-            }
-            else {
-                view.setNumThreads(1);
-            }
+
+            view.setNumThreads(numThreads);
+
             view.regenerate(false);
         }
     }
