@@ -23,6 +23,8 @@
 function OptionsToolBar() {
 }
 
+OptionsToolBar.includeBasePath = includeBasePath;
+
 OptionsToolBar.normalizeSeparators = function(action) {
     // remove double separators:
     var optionsToolBar = EAction.getOptionsToolBar();
@@ -71,13 +73,57 @@ OptionsToolBar.init = function(basePath) {
     var optionsToolBar = EAction.getOptionsToolBar();
 };
 
+OptionsToolBar.initStyle = function() {
+    var optionsToolBar = EAction.getOptionsToolBar();
+
+    var iconLabel = optionsToolBar.findChild("Icon");
+
+    // style of label with current tool icon:
+    if (!RSettings.hasCustomStyleSheet()) {
+        var w = optionsToolBar.iconSize.width() < 20 ? 1 : 2;
+        if (RSettings.hasDarkGuiBackground()) {
+            iconLabel.styleSheet =
+                "QLabel {"
+                + "border-radius: %1px; ".arg(w*3)
+                + "background-color: "
+                + "    qlineargradient(spread:pad, "
+                + "        x1: 0, y1: 0, "
+                + "        x2: 0, y2: 1, "
+                + "        stop: 0 #555555, "
+                + "        stop: 0.5 #606060, "
+                + "        stop: 1 #555555"
+                + "    ); "
+                + "border: %1px solid #a0a0a0;".arg(w)
+                + "margin: %1px %1px %1px %1px;".arg(w)
+                + "}";
+        }
+        else {
+            iconLabel.styleSheet =
+                "QLabel {"
+                + "border-radius: %1px; ".arg(w*3)
+                + "background-color: "
+                + "    qlineargradient(spread:pad, "
+                + "        x1: 0, y1: 0, "
+                + "        x2: 0, y2: 1, "
+                + "        stop: 0 rgba(255,255,255,0), "
+                + "        stop: 0.5 rgba(255,255,255,192), "
+                + "        stop: 1 rgba(255,255,255,0) "
+                + "    ); "
+                + "border: %1px solid #8f8f8f;".arg(w)
+                + "margin: %1px %1px %1px %1px;".arg(w)
+                + "}";
+        }
+    }
+};
+
 OptionsToolBar.postInit = function(basePath) {
     var optionsToolBar = EAction.getOptionsToolBar();
-    var sh = EAction.getMainWindow().findChild("FileToolBar").sizeHint;
 
     // fixed height to prevent FS#652 (can happen at least on Windows and Linux):
-    var h = RSettings.getIntValue("ToolBar/IconSize", 38);
-    optionsToolBar.setFixedHeight(h+6);
+    var ftb = EAction.getMainWindow().findChild("FileToolBar");
+    if (!isNull(ftb)) {
+        optionsToolBar.setFixedHeight(Math.min(ftb.sizeHint.height(), ftb.sizeHint.width()));
+    }
 
     var flags = new Qt.ToolBarAreas(Qt.TopToolBarArea | Qt.BottomToolBarArea);
     optionsToolBar.setAllowedAreas(flags);
@@ -86,32 +132,24 @@ OptionsToolBar.postInit = function(basePath) {
     //optionsToolBar.floatable = false;
 
     var iconLabel = new QLabel(optionsToolBar);
+    //var iconLabel = new QToolButton(optionsToolBar);
     iconLabel.objectName = "Icon";
     iconLabel.alignment = Qt.AlignCenter;
     iconLabel.setContentsMargins(6, 0, 6, 0);
+    //iconLabel.setFrameStyle(QFrame.Panel | QFrame.Sunken);
 
-    if (!RSettings.hasCustomStyleSheet()) {
-        var w = optionsToolBar.iconSize.width() < 20 ? 1 : 2;
-        iconLabel.styleSheet =
-            "QLabel {"
-            + "border-radius: %1px; ".arg(w*3)
-            + "background-color: "
-            + "    qlineargradient(spread:pad, "
-            + "        x1: 0, y1: 0, "
-            + "        x2: 0, y2: 1, "
-            + "        stop: 0 rgba(255,255,255,0), "
-            + "        stop: 0.5 rgba(255,255,255,192), "
-            + "        stop: 1 rgba(255,255,255,0) "
-            + "    ); "
-            + "border: %1px solid #8f8f8f;".arg(w)
-            + "margin: %1px %1px %1px %1px;".arg(w)
-            + "}";
-    }
+    OptionsToolBar.initStyle();
 
     // avoid empty label after startup, before initializing new document:
     OptionsToolBar.setIcon("scripts/Reset/Reset.svg");
+
     optionsToolBar.addWidget(iconLabel);
     optionsToolBar.addSeparator();
+
+    var appWin = RMainWindowQt.getMainWindow();
+    var pl = new RPaletteListenerAdapter();
+    appWin.addPaletteListener(pl);
+    pl.paletteChanged.connect(OptionsToolBar.initStyle);
 };
 
 /**
@@ -133,10 +171,33 @@ OptionsToolBar.setIcon = function(fileNameOrIcon) {
     var w = optionsToolBar.iconSize.width()*f;
     var h = optionsToolBar.iconSize.height()*f;
 
+
+//    var iconSize = RSettings.getIntValue("CadToolBar/IconSize", 32);
+//    iconSize = Math.max(iconSize, RSettings.getIntValue("ToolBar/IconSize", 32));
+//    iconSize = Math.max(iconSize, RSettings.getIntValue("CadToolMatrix/IconSize", 24));
+
+//    // retina icons:
+//    if (RSettings.getDevicePixelRatio()>1) {
+//        iconSize*=RSettings.getDevicePixelRatio();
+//    }
+
+//    var pm = new QPixmap(iconSize,iconSize);
+//    //pm.fill(Qt.transparent);
+//    var p = new QPainter();
+//    p.begin(pm);
+//    var renderer = new QSvgRenderer(OptionsToolBar.includeBasePath + "/IconBackground.svg");
+//    renderer.render(p, new QRectF(0, 0, iconSize, iconSize));
+
     if (isOfType(fileNameOrIcon, QIcon)) {
         iconLabel.pixmap = fileNameOrIcon.pixmap(new QSize(w, h));
+        //iconLabel.icon = fileNameOrIcon;
     }
     else {
         iconLabel.pixmap = new QIcon(autoPath(fileNameOrIcon)).pixmap(new QSize(w, h));
+        //iconLabel.icon = new QIcon(autoPath(fileNameOrIcon));
     }
+
+//    p.end();
+
+
 };
